global LastChannel
global PauseTime
global TestAfterId
global TimerContinue
global Abort
global TestDebug

# 15 minutes
set PauseTime [expr 15 * 60 * 1000]
# 5 seconds
set PauseTime [expr 5 * 1000]

set LastChannel 5

set to "/home/scl/specs/porgamrays"
set savedir "Cal-Ch"

set hpath "/hist"

set TestAfterId ""
set TimerContinue 0
set Abort 0


set TestDebug 1

proc InsertDebug {args} {
   global TestDebug
   if {$TestDebug} {insert-debug [first $args]}
}

proc Zero {sp} {
   global hpath
   InsertDebug "Doing Zero $sp"
   set z [catch {eg "(spectrum :path '$hpath' :name '$sp' !zero)"} m]
   if {$z!=0} then {insert-log "spectrum $s not zeroed - [eg "(sys :error $m ?message)"]"}
}

proc ZeroChannel {s} {
   InsertDebug "Zeroing spectrum channel $s"
   Zero Ch$s
}

proc ZeroAll {} {
   global LastChannel
   InsertDebug "Zeroing all spectra"
   Zero "Stat"
   Zero "Rate"
   for {set s 0} {$s <= $LastChannel} {incr s} {
      ZeroChannel $s
   }
}

proc Go {args} {
   InsertDebug "Doing Go"
   inform sys "click-da 1"
}

proc Stop {args} {
   InsertDebug "Doing Stop"
   inform sys "click-da 0"
}

proc Pause {args} {
   global %%Waiting 
   global PauseTime TestAfterId Abort TimerContinue

   set TestAfterId [after $PauseTime AfterProc]
   InsertDebug "Pausing"
   set TimerContinue 0
   WaitForUser "Waiting for Pause Timer"
   catch {after cancel $TestAfterId}
   if {[set %%Waiting] == -2} then {
      set Abort 1
   } else {
      InsertDebug "Continuing"
   }
}

proc Save {s} {
   global hpath to savedir LastChannel

   set now [date +%Y%h%d-%H.%M.%S]
   set todir [file join $to $savedir$s $now]
   InsertDebug "Saving spectra for channel $s in $todir"
   :file mkdir $todir

   set from $hpath

   set slist Stat
   for {set s 0} {$s <= $LastChannel} {incr s} {
      lappend slist Ch$s
   }
   set z [catch {eval exec copy-spectra $from $todir $slist} m]

   set slist {}
   for {set s 0} {$s <= $LastChannel} {incr s} {
      lappend slist [file join $hpath Ch$s]
   }
   set z [catch {eval exec total-counts -spectrum $slist} m]
   if {!$z} {
      set cf [file join $todir Counts]
      set z [catch {set f [open $cf w]} x]
      if {!$z} {
         puts $f $m
         close $f
      } else {
         insert-log "spectrum counts file $cf not created"
      }
   } else {
      insert-log "could not get total-counts for spectra in $hpath"
   }
}

proc AfterProc {} {
   global TimerContinue
   InsertDebug "Continuing after timer"
   set TimerContinue 1
   ContinueScript
}

proc RunLoop {args} {
   global LastChannel Abort TimerContinue

   Stop
   set Abort 0
   for {set s 0} {$s <= $LastChannel} {incr s} {
      ZeroAll
      Go
      Pause
      Stop
      Save $s
      if {$Abort} then {
         InsertDebug "Aborting"
         break
      } else {
          if {!$TimerContinue} {InsertDebug "User clicked \"Continue\""}
      }
   }
}

RunLoop

InsertDebug "Loop Done - exiting"
