#   PorgamRays Setup aand Control

set frame  PorgamRays
set title "PorgamRays Setup"
set icon   PorgamRays

set started 0
set verbose 0

set inputcolour  green4
set outputcolour red
set insertcolour blue
set bkgcolour    azure

set inputprompt  ""
set outputprompt ""
set insertprompt ">>"


set XPRAYSNUCAMAccessServerHost  "localhost"

#    ASIC register values

set PreAmpGain 0
set ShapingTime 2
set PreAmpBias 6

set RiseTime 0
set TimeStamp 0
set ChipBiasCurrent 9

for {set i 0} {$i <= 101} {incr i} {set MASKRegister($i) 0}
for {set i 0} {$i <= 100} {incr i} {set CALRegister($i) 0}




proc bgerror err {
    global errorInfo tkerrorInfo tkskiperror
    upvar #0 MIDAS_SOAPFaultCode FaultCode
    upvar #0 MIDAS_SOAPFaultString FaultString
    upvar #0 MIDAS_SOAPErrorCode ErrorCode
    upvar #0 MIDAS_SOAPErrorInfo ErrorInfo

    if {$tkskiperror == 1} then {set tkskiperror 0; return}
    set tkerrorInfo $errorInfo

    if {[grab current .] != ""} then {grab release [grab current .]}

    catch {:modify [self frame-name] :busy T}
    if {![info exists FaultCode] || $FaultCode == "NONE"} {
        set z [catch {midas_dialog [self frame-name].tkerrorDialog "Error Detected in Tcl Script" \
                "Task '[Get_My_Name]' detected error in Tcl script\n\n$err" "error" 0 "continue" "Stack Trace"} button]
    } else {
        set z [catch {midas_dialog [self frame-name].tkerrorDialog "Error Detected in Tcl Script" \
                "Task '[Get_My_Name]' detected error in Tcl script\n\n$FaultCode\n$FaultString\n$ErrorCode\n$ErrorInfo" \
                       "error" 0 "continue"} button]
    }
    catch {:modify [self frame-name] :busy F}

    if {$z != 0} then {
       puts stderr "midas_dialog: [self frame-name].tkerrorDialog: $button"
       destroy [self frame-name].tkerrorDialog
       return
    }

    if {$button != 0} then {tkStackTrace}
}




proc Set_Register {Addr Vl}  {
    global errorCode errorInfo

        set rc [catch {XPRAYSNUCAMAccessClient__Set $Addr $Vl} m]
        if {$rc != 0} {
            InsertLog  "Set returned with an error\n$m\n$errorCode\n$errorInfo"
            return -1
        } else {
            return $m
        }
}

proc Get_Register {Addr}  {
    global errorCode errorInfo

        set rc [catch {XPRAYSNUCAMAccessClient__Get $Addr} m]
        if {$rc != 0} {
            InsertLog  "Get returned with an error\n$m\n$errorCode\n$errorInfo"
            return -1
        } else {
            return $m
        }
}

proc InsertLog {s} {:modify LOG :usingtag taginsert :strings $s\n}


proc click arg {
    global PreAmpGain  ShapingTime  PreAmpBias  RiseTime  TimeStamp  ChipBiasCurrent
 
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]

   if {$c==":quit"} then {click-quit; return}
   switch $w {

      RTT	{Write_DAC A $v; after 200 Redisplay_DACSettings}
      ET 	{Write_DAC B $v; after 200 Redisplay_DACSettings}
      CPL	{Write_DAC C $v; after 200 Redisplay_DACSettings}

      PAG       {set PreAmpGain $v; Write_ASICSettings; Redisplay_ASICSettings}
      ST        {set ShapingTime $v; Write_ASICSettings; Redisplay_ASICSettings}
      PAB       {set PreAmpBias $v; Write_ASICSettings; Redisplay_ASICSettings}
      RT        {set RiseTime $v; Write_ASICSettings; Redisplay_ASICSettings}
      TS        {set TimeStamp $v; Write_ASICSettings; Redisplay_ASICSettings}
      CBC       {set ChipBiasCurrent $v; Write_ASICSettings; Redisplay_ASICSettings}

      CALENABLE {Set_Register 0x0208 2}
      CALRATE   {Set_Register 0x020a $v}
   
      default {
         set p "click-[lowercase $w]"
         if {[info procs $p] == "$p"} then {$p $v} else {set-footer "no handler for widget $w"}
      }
   }
}


proc make_register {widget} {

     :layout H; :row-gap 0; :next-row T

     for {set j 0} {$j < 11} {incr j} {

        :canvas [set widget]DECADE$j  :w 320 :h 35

        for {set i 0} {$i < 10} {incr i} {
           set y 20
           set x [expr $i * 30  + 20]
           if {$i > 4} then {incr x 10}
           entry [:path [set widget]DECADE$j].bit$i -relief flat -width 2 -justify right
           [:path [set widget]DECADE$j].bit$i insert 0 " "
           :draw window :tag [set widget]bit$i :value $x $y -window [:path [set widget]DECADE$j].bit$i 

           bind [:path [set widget]DECADE$j].bit$i <ButtonPress-1>  "set_register_bit $widget $j $i"
        }

        set l [expr 100 - ($j*10)]
        :message [set widget]M$j  :label $l  :w 4

        :next-row T; :vskip 10
     }
     :row-gap 10
     :vskip 10
}

proc show_register {widget} {
    global env
    global [set widget]Register

   if {$widget == "MASK"} {set maxbit 101} else {set maxbit 100}

     set bit 0

     for {set j 0} {$j < 11} {incr j} {
        for {set i 0} {$i < 10} {incr i} {
           set bit [expr (109 - ($j*10)) - $i]
           [:path [set widget]DECADE$j].bit$i delete 0 end
           if {$bit > $maxbit} {[:path [set widget]DECADE$j].bit$i configure -bg  black; continue}
           [:path [set widget]DECADE$j].bit$i insert 0 [set [set widget]Register($bit)]
           if {[set [set widget]Register($bit)] == 0} {[:path [set widget]DECADE$j].bit$i configure -bg  $env(tkcheckbuttonbackground)} \
               else {[:path [set widget]DECADE$j].bit$i configure -bg  $env(tkcheckbuttonselectcolor)}
        }
     }
}

proc set_register_bit {widget j i} {
    global env
    global [set widget]Register

    if {$widget == "MASK"} {set maxbit 101} else {set maxbit 100}

    set bit [expr (109 - ($j*10)) - $i]

    if {$bit > $maxbit} {return}

    set vl  [set [set widget]Register($bit)]
    set [set widget]Register($bit) [expr ($vl + 1) & 1]

    [:path [set widget]DECADE$j].bit$i delete 0 end
    [:path [set widget]DECADE$j].bit$i insert 0 [set [set widget]Register($bit)]
    if {[set [set widget]Register($bit)] == 0} {[:path [set widget]DECADE$j].bit$i configure -bg  $env(tkcheckbuttonbackground)} \
        else {[:path [set widget]DECADE$j].bit$i configure -bg  $env(tkcheckbuttonselectcolor)}

}

proc click-setmask {args} {set_bitregister MASK}
proc click-setcal  {args} {set_bitregister CAL}

proc set_bitregister {widget} {
    global errorCode errorInfo
    global [set widget]Register

    if {$widget == "MASK"} {set maxbit 101} else {set maxbit 100}

    set Vl ""
    for {set bit $maxbit} {$bit >= 0} {incr bit -1} {append Vl " [set [set widget]Register($bit)]"}

    if {$widget == "MASK"} {set Address 0x0100} else {set Address 0x0200}

        set rc [catch {XPRAYSNUCAMAccessClient__SetBitRegister $Address $Vl} m]
        if {$rc != 0} {
            InsertLog  "SetBitRegister returned with an error\n$m\n$errorCode\n$errorInfo"
            return -1
        } else {
            return $m
        }
}

proc do_Reset {} {
     set r [Get_Register 0x0208]
     Set_Register 0x0208 [expr ($r & 0x7f) + 0x80]     ;# toggle bit 7
     Set_Register 0x0208 $r
}

proc make-frame {} {
   global env
   global inputcolour outputcolour insertcolour bkgcolour

set rows 16
set cols 80

  :frame [self frame-name] :label "[self frame-title]" :icon-name "[self icon-title]"

   :panel P0 :layout T :padx 5 :pady 5
    :layout H; :col-gap 10

     :slider RTT  :label "Rise Time Threshold:" :min 0 :max 255 :value 0
     :slider ET   :label "Energy Threshold:" :min 0 :max 255 :value 0
     :slider CPL  :label "Calibration Pulse Level:" :min 0 :max 255 :value 0

     :next-row T

     :message M1 :label "ASIC Settings" :bold T
     :next-row T
     :choice PAG :label "PreAmp Gain" :strings [list Low High]
     :choice-stack ST :label "Shaping Time (us)" :strings [list - 0.5 1.0 1.5 2.0 2.5 3.0 2.5 4.0 4.5 5..0 5.5 6.0 6.5 7.0 7.5]
     :choice-stack PAB :label "PreAmp Bias (nA)" :strings [list 0.25 0.5 0.75 1 1 2 3 4]
     :choice RT :label "Rise Time (MHz)" :strings [list 32 16]
     :choice TS :label "Time Stamp (MHz)" :strings [list 1 2 4 8]
     :choice-stack CBC :label "Chip Bias Current (uA)" :strings [list 450 433 417 400 383 367 350 333 317 300 283 \
                       267 250 233 217 200 250 233 217 200 183 167 150 133 117 100 83 67 50 33 17]

   :panel P1L :layout L :padx 50 :pady 5
    :layout V; :col-gap 10

     :message M2 :label "Mask Register" :bold T
      make_register MASK
     :button SETMASK :label "Set Mask Register" :w 24
     :layout H
     :hskip 20
     :button RESET  :command do_Reset

   :panel P1R :layout L :padx 0 :pady 5
    :layout V; :col-gap 10

     :message M3 :label "Calibration Register" :bold T
      make_register CAL
     :button SETCAL :label "Set Calibration Register" :w 24
     :layout H
     :button CALENABLE  :label "Enable Calibration"
     :slider CALRATE    :label "Pulse Rate" :min 0 :max 65536

   :panel P2 :layout B :padx 5 :pady 5  :after FOOTER
    :galley LOG :rows $rows :cols $cols :font fixed

   :panel LAST :layout B :padx 5 :pady 5 :after FOOTER
    :layout H; :col-gap 20

     :button REDISPLAY :command do_redisplay
     :hskip 100
     :button CLEAR   :label "Empty Log Window" :command ":modify LOG :empty T"

   open-frame

  :galley_size LOG $rows $cols

    :modify LOG :createtag "taginput  -foreground $inputcolour  -background $bkgcolour"
    :modify LOG :createtag "tagoutput -foreground $outputcolour -background $bkgcolour"
    :modify LOG :createtag "taginsert -foreground $insertcolour -background $bkgcolour"

    :show T

}

proc Save_Options {} {
   global Options


   update_Options
}


proc start args {
   global started env Options

   if {$started == 0} then {make-frame} else {open-frame}
   set started 1

#    load application options and establish default values for any not defined

   load_Options

   if {![info exists Options(WebServer)]}     {set Options(WebServer) "localhost"}

   update_Options

   click-server $Options(WebServer)

}

proc click-server {v} {
    global XPRAYSNUCAMAccessServerHost
    global started

    Save_Options

    set XPRAYSNUCAMAccessServerHost $v

    package require XPRAYSNUCAMAccessClient
    loadsource XPRAYSNUCAMAccessClient     ;# better solution needed

    InsertLog "Initialised XPRAYSNUCAM Web Service at $XPRAYSNUCAMAccessServerHost"

    set started 2

   do_redisplay
}

proc I2C_busy {} {

#    check that the I2C bus is not busy and ready to accept an operation
     
    set vl [Get_Register 9]     ;# read Status register
    if {$vl == -1} {midas-warning "Error reading I2C Status register"; return -1}
    if {[expr $vl & 1] != 0} {midas-warning "I2C busy"; return -1}
    if {[expr $vl & 8] == 8} {    ;# last operation failed
        Set_Register 8 1
        set vl [Get_Register 9]
        if {$vl == -1 || [expr $vl & 9] != 0} {midas-warning "I2C problems event after reset"; return -1}
    }
    
    return 0
}

proc I2C_check {} {

#    check result of last operation on the I2C bus

    set vl [Get_Register 9]     ;# read Status register
    if {$vl == -1} {midas-warning "Error reading I2C Status register"; return -1}
    if {[expr $vl & 1] != 0} {
        after 2       ;# busy - allow a short delay
        set vl [Get_Register 9]     ;# read Status register
        if {$vl == -1} {midas-warning "Error reading I2C Status register"; return -1}
        if {[expr $vl & 1] != 0} {midas-warning "I2C bus busy even after delay"; return -1}
    }
    if {[expr $vl & 8] == 8} {midas-warning "I2C bus operation failed"; return -1}

    return 0
}

proc Write_DAC {w vl} {

#    write DAC $w

     switch $w {
         A   {set r 4}
         B   {set r 5}
         C   {set r 6}
         D   {set r 7}
     }

     if {[I2C_busy] != 0} {return -1}

     set vl [Set_Register $r [expr 0x2000 + (($vl & 255) << 4)]]
     if {$vl == -1} {midas-warning "Error writing DAC $w"; return -1}

     if {[I2C_check] != 0} {return -1}

     return 0
}

proc Read_DAC {w} {

#    read DAC $w

     switch $w {
         A   {set r 4}
         B   {set r 5}
         C   {set r 6}
         D   {set r 7}
     }

     if {[I2C_busy] != 0} {return -1}

     set vl [Get_Register $r]
     if {$vl == -1} {midas-warning "Error reading DAC $w"; return -1}

     if {[I2C_check] != 0} {return -1}

     set vl [Get_Register 10]    ;# get the actual data
     if {$vl == -1} {midas-warning "Error reading DAC $w"; return -1}
 
     set vl [expr ($vl >> 4 ) & 255 ]    ;# extract the byte from the word

     return $vl
}

proc Redisplay_DACSettings {} {

    :modify RTT :value [Read_DAC A]
    :modify ET  :value [Read_DAC B]
    :modify CPL :value [Read_DAC C]

}

proc Read_ASICregister {w} {

#    read ASIC $w   (w = 0 or 1)

     if {[I2C_busy] != 0} {return -1}

     set vl [Get_Register $w]
     if {$vl == -1} {midas-warning "Error reading ASIC register $w"; return -1}

     if {[I2C_check] != 0} {return -1}

     set vl [Get_Register 10]    ;# get the actual data
     if {$vl == -1} {midas-warning "Error reading ASIC register $w"; return -1}

     return $vl
}

proc Write_ASICregister {w vl} {

#    write ASIC $w   (w = 0 or 1)

     if {[I2C_busy] != 0} {return -1}

     set vl [Set_Register $w $vl]
     if {$vl == -1} {midas-warning "Error writing ASIC register $w"; return -1}

     if {[I2C_check] != 0} {return -1}

     return 0
}

proc Read_ASICSettings {} {
    global PreAmpGain  ShapingTime  PreAmpBias  RiseTime  TimeStamp  ChipBiasCurrent 

    set vl [Read_ASICregister 0]
    if {$vl == -1} {return}

    set PreAmpBias [expr $vl & 7]
    set ShapingTime [expr ($vl >> 3) & 15]
    set PreAmpGain [expr ($vl >> 7) & 1]

    set vl [Read_ASICregister 1]
    if {$vl == -1} {return}

    set ChipBiasCurrent [expr $vl & 0x1f]
    set TimeStamp [expr ($vl >> 5) & 3]
    set RiseTime [expr ($vl >> 7) & 1]

}

proc Write_ASICSettings {} {
    global PreAmpGain  ShapingTime  PreAmpBias  RiseTime  TimeStamp  ChipBiasCurrent

    set Pre2 [expr $PreAmpBias & 4]
    set Pre10 [expr (~($PreAmpBias & 3)) & 3]

    Write_ASICregister 0 [expr ($PreAmpGain << 7) + ($ShapingTime << 3) + $Pre2 + $Pre10]
    Write_ASICregister 1 [expr ($RiseTime << 7) + ($TimeStamp << 5) + ((~$ChipBiasCurrent) & 0x1f)]

}

proc Redisplay_ASICSettings {} {
    global PreAmpGain  ShapingTime  PreAmpBias  RiseTime  TimeStamp  ChipBiasCurrent

     Read_ASICSettings

     :modify PAG :value $PreAmpGain
     :modify ST  :value $ShapingTime
     :modify PAB :value $PreAmpBias
     :modify RT  :value $RiseTime
     :modify TS  :value $TimeStamp
     :modify CBC :value $ChipBiasCurrent 

}

proc do_redisplay {} {
    Redisplay_ASICSettings
    Redisplay_DACSettings
    show_register MASK
    show_register CAL
    :modify CALRATE :value [Get_Register 0x020a]
}



#    The Options are experiment specific and normally used by SYS
#    However the same code as for the session preferences is used which allows for wider use in the future.

proc get_Option_file {} {
    global env

#    look for the Option file
    set pd $env(EXPERIMENTBASE)
    if {![file exists $pd]} then {
         insert-log "$pd does not exist - Join an experiment!"
         return {}
    }
    if {![file isdirectory $pd]} then {
         insert-log "$pwd is not useable - it is not a directory"
         return {}
    }
    set pf [file join $pd MidasOptions]
    set z [catch {set f [open $pf r]} m]
    if {$z != 0} then {
        set z [catch {set f [open $pf w]} m]
        if {$z != 0} then {
            insert-log "Unable to access $pf - $m"
            return 1
        }
        close $f
        return {}
    }

#    and read it
    set lines [split [read $f] "\n"]
    close $f

    return $lines
}

proc put_Option_file {lines} {
   global env

#    look for the Option file
    set pd $env(EXPERIMENTBASE)
    if {![file exists $pd]} then {
         insert-log "$pd does not exist - Join an experiment!"
         return {}
    }
    if {![file isdirectory $pd]} then {
         insert-log "$pwd is not useable - it is not a directory"
         return {}
    }
    set pf [file join $pd MidasOptions]
    set z [catch {set f [open $pf w]} m]
    if {$z != 0} then {
        insert-log "Unable to access $pf - $m"
        return 1
    }

    foreach item $lines {
      if {$item != ""} {puts $f $item}
    }

    close $f
    return 0
}

#    save all Options for the current task
proc update_Options {} {
   global env Options

   set lines [get_Option_file]
   set key [Get_My_Name]
   set range [find-keyword $lines $key]
   if {[llength $range] > 0} then {
       set lines [lreplace $lines [first $range] [expr [second $range] + 1]]
   }

   lappend lines "## $key"
   foreach item [array names Options] {
       lappend lines "$item $Options($item)"
   }
   lappend lines "##"

   set z [put_Option_file $lines]
   return $z
}

#    load all Options for the current task
proc load_Options {} {
   global env Options

   set lines [get_Option_file]
   set key [Get_My_Name]
   set range [find-keyword $lines $key]

   if {[llength $range] > 1} then {
       set stuff [lrange $lines [expr [first $range]+1] [second $range]]
       foreach item $stuff {
           set Options([head $item]) [tail $item]
       }
   }
}

#    return task Options
proc get_Options {task} {
   global env

   set lines [get_Option_file]
   set key $task
   set range [find-keyword $lines $key]

   set Options {}

   if {[llength $range] > 1} then {
       set stuff [lrange $lines [expr [first $range]+1] [second $range]]
       foreach item $stuff {
           lappend Options [list [first $item] [second $item]]
       }
   } else {
       lappend Options [list Options.filename  "[file join $env(HOME) MidasOptions Options]"]
   }
   return $Options
}

